; Support for controls on PUDed module windows


DEFAULT_CONTROL_STYLE   equ WS_CLIPSIBLINGS or WS_CHILD or WS_VISIBLE or WS_TABSTOP
DEFAULT_CONTROL_EXSTYLE equ WS_EX_CLIENTEDGE or WS_EX_ACCEPTFILES

STYLE_BUTTON          equ BS_PUSHBUTTON
STYLE_CHECKBOX        equ BS_AUTOCHECKBOX or BS_NOTIFY
STYLE_RADIOCHECK      equ BS_AUTORADIOBUTTON                or BS_NOTIFY
STYLE_TEXTEDIT        equ WS_BORDER       or ES_AUTOHSCROLL or ES_LEFT
STYLE_COMBOBOX        equ WS_VSCROLL      or CBS_DROPDOWN   or CBS_HASSTRINGS
STYLE_LISTBOX         equ WS_VSCROLL      or LBS_NOTIFY     or WS_BORDER
STYLE_TEXT            equ WS_CLIPSIBLINGS or WS_CHILD or WS_VISIBLE or SS_NOTIFY

Strikeout             equ   1
Underline             equ   2
Italic                equ   4
Bold                  equ   8



.data?
  ControlStringBuffer db 200h dup (?)

.data

  szBUTTON    db "BUTTON"   ,0
  szEDIT      db "EDIT"     ,0
  szCOMBOBOX  db "COMBOBOX" ,0
  szLISTBOX   db "LISTBOX"  ,0
  szSTATIC    db "STATIC"   ,0

  BUTTONclass     dd DEFAULT_CONTROL_STYLE or STYLE_BUTTON
		  dd 0
		  dd OFFSET szBUTTON

  TEXTclass       dd STYLE_TEXT
		  dd 0
		  dd OFFSET szSTATIC

  CHECKBOXclass   dd DEFAULT_CONTROL_STYLE or STYLE_CHECKBOX
		  dd 0
		  dd OFFSET szBUTTON

  RADIOCHECKclass dd DEFAULT_CONTROL_STYLE or STYLE_RADIOCHECK
		  dd WS_EX_TRANSPARENT
		  dd OFFSET szBUTTON

  TEXTEDITclass   dd DEFAULT_CONTROL_STYLE or STYLE_TEXTEDIT
		  dd DEFAULT_CONTROL_EXSTYLE
		  dd OFFSET szEDIT

  COMBOBOXclass   dd DEFAULT_CONTROL_STYLE or STYLE_COMBOBOX
		  dd DEFAULT_CONTROL_EXSTYLE
		  dd OFFSET szCOMBOBOX

  LISTBOXclass    dd DEFAULT_CONTROL_STYLE or STYLE_LISTBOX
		  dd DEFAULT_CONTROL_EXSTYLE
		  dd OFFSET szLISTBOX


  PUSHBUTTON      dd OFFSET BUTTONclass
  TEXT            dd OFFSET TEXTclass
  TEXTEDIT        dd OFFSET TEXTEDITclass
  CHECKBOX        dd OFFSET CHECKBOXclass
  RADIOCHECK      dd OFFSET RADIOCHECKclass
  COMBOBOX        dd OFFSET COMBOBOXclass
  LISTBOX         dd OFFSET LISTBOXclass


.code

  ; ==================================
  ; == Control Interaction Routines ==
  ; ==================================
  ;
  ; A few simple procs to take some of the grunt work
  ; out of working with controls. 
  ;
  ; Routines the retrieve strings all use the same buffer
  ; so strings that need to be preserved need to be copied
  ; elsewhere before continuing.
  ;
  ; These are not, nor are they intended to be a complete
  ; wrapper of any sort. They don't do any error handling.
  ; They are here as much for demonstration purposes as
  ; actual use.
  ;



; == General ==

getText proc hControl:DWORD
  ; ---------------------------
  ; get the text from a control
  ; ---------------------------
  push ecx
  push edx
  
    mov eax,OFFSET ControlStringBuffer
    mov BYTE ptr[eax],0
    push eax
      push 1FFh
      push eax
      push hControl
      call GetWindowTextA
    pop eax
  
  pop edx
  pop ecx
  ret
getText endp

setText proc hControl:DWORD, pszText:DWORD
  ; -------------------------
  ; set the text in a control
  ; -------------------------
  push ecx
  push edx
    push pszText
    push hControl
    call SetWindowTextA
  pop edx
  pop ecx
  ret
setText endp


; == Text / Edit Text ==
;
;   use getText  and  setText


; == Check Box ==

getCheck proc hCheckbox:DWORD
  ;--------------------------------------
  ; return the state of a checkbox or radiocheck
  ;--------------------------------------
  push 0
  push 0
  push BM_GETCHECK
  push hCheckbox
  call doSendMessage
  ret
getCheck endp

setCheck proc hCheckbox:DWORD, state:DWORD
  ;--------------------------------------
  ; set the state of a checkbox or radiocheck
  ;--------------------------------------
  push 0
  push state
  push BM_SETCHECK
  push hCheckbox
  call doSendMessage
  ret
setCheck endp





; == List Box ==


listAdd proc hListbox:DWORD, pszString:DWORD
  ;---------------------------
  ; add a string to a listbox
  ;---------------------------
  push pszString
  push 0
  push LB_ADDSTRING
  push hListbox
  call doSendMessage
  ret
listAdd endp

listInsert proc hListbox:DWORD, ndx:DWORD, pszString:DWORD
  ;--------------------------------------
  ; insert a string into a listbox at position ndx
  ;--------------------------------------
  dec ndx
  push pszString
  push ndx
  push LB_INSERTSTRING
  push hListbox
  call doSendMessage
  ret
listInsert endp

listDelete proc hListbox:DWORD, ndx:DWORD, pszString:DWORD
  ;--------------------------------------
  ; delete the string at position ndx from a listbox
  ;--------------------------------------
  dec ndx
  push pszString
  push ndx
  push LB_DELETESTRING
  push hListbox
  call doSendMessage
  ret
listDelete endp

listReset proc hListbox:DWORD
  ;--------------------------------------
  ; delete all contents from a listbox
  ;--------------------------------------
  push 0
  push 0
  push LB_RESETCONTENT
  push hListbox
  call doSendMessage
  ret
listReset endp

listCount proc hListbox:DWORD
  ;--------------------------------------
  ; return the number of strings in a listbox
  ;--------------------------------------
  push 0
  push 0
  push LB_GETCOUNT
  push hListbox
  call doSendMessage
  ret
listCount endp

listIndex proc hListbox:DWORD
  ;--------------------------------------
  ; return the number of the selected string in a listbox
  ;--------------------------------------
  push 0
  push 0
  push LB_GETCURSEL
  push hListbox
  call doSendMessage
  inc eax
  ret
listIndex endp

listSetIndex proc hListbox:DWORD, ndx:DWORD
  ;--------------------------------------
  ; set the selected string in a listbox
  ;--------------------------------------
  dec ndx
  push 0
  push ndx
  push LB_SETCURSEL
  push hListbox
  call doSendMessage
  ret
listSetIndex endp

listText proc hListbox:DWORD, ndx:DWORD
  ;--------------------------------------
  ; return the string at position ndx in a listbox
  ;--------------------------------------
  ; if ndx=0 return the selected string
  mov eax,OFFSET ControlStringBuffer
  mov BYTE ptr[eax],0
  cmp ndx,0
  je @F
    dec ndx
    push eax
      push eax
      push ndx
      push LB_GETTEXT
      push hListbox
      call doSendMessage
    pop eax
    ret
  @@:
  push hListbox
  call listIndex
  test eax,eax
  jz @F
    push eax
    push hListbox
    call listText
  @@:
  ret
listText endp




; == Combo Box ==


comboAdd proc hCombobox:DWORD, pszString:DWORD
  ;--------------------------------------
  ; add a string to a combobox
  ;--------------------------------------
  push pszString
  push 0
  push CB_ADDSTRING
  push hCombobox
  call doSendMessage
  ret
comboAdd endp

comboInsert proc hCombobox:DWORD, ndx:DWORD, pszString:DWORD
  ;--------------------------------------
  ; insert a string into a combobox at position ndx
  ;--------------------------------------
  dec ndx
  push pszString
  push ndx
  push CB_INSERTSTRING
  push hCombobox
  call doSendMessage
  ret
comboInsert endp

comboDelete proc hCombobox:DWORD, ndx:DWORD, pszString:DWORD
  ;--------------------------------------
  ; delete the string at position ndx from a combobox
  ;--------------------------------------
  dec ndx
  push pszString
  push ndx
  push CB_DELETESTRING
  push hCombobox
  call doSendMessage
  ret
comboDelete endp

comboReset proc hCombobox:DWORD
  ;--------------------------------------
  ; delete all contents from a combobox
  ;--------------------------------------
  push 0
  push 0
  push CB_RESETCONTENT
  push hCombobox
  call doSendMessage
  ret
comboReset endp

comboCount proc hCombobox:DWORD
  ;--------------------------------------
  ; return the number of strings in a combobox
  ;--------------------------------------
  push 0
  push 0
  push CB_GETCOUNT
  push hCombobox
  call doSendMessage
  ret
comboCount endp

comboIndex proc hCombobox:DWORD
  ;-------------------------------------------------------
  ; return the number of the selected string in a combobox
  ;-------------------------------------------------------
  push 0
  push 0
  push CB_GETCURSEL
  push hCombobox
  call doSendMessage
  inc eax
  ret
comboIndex endp

comboSetIndex proc hCombobox:DWORD, ndx:DWORD
  ;--------------------------------------
  ; set the selected string in a combobox
  ;--------------------------------------
  dec ndx
  push 0
  push ndx
  push CB_SETCURSEL
  push hCombobox
  call doSendMessage
  ret
comboSetIndex endp

comboListText proc hCombobox:DWORD, ndx:DWORD
  ; return the string at position ndx in the
  ; listbox attached to a combobox 
  ; if ndx=0 return the selected string
  mov eax,OFFSET ControlStringBuffer
  mov BYTE ptr[eax],0
  cmp ndx,0
  je @F
    dec ndx
    push eax
      push eax
      push ndx
      push CB_GETLBTEXT
      push hCombobox
      call doSendMessage
    pop eax
    ret
  @@:
  push hCombobox
  call comboIndex
  test eax,eax
  jz @F
    push eax
    push hCombobox
    call comboListText
  @@:
  ret
comboListText endp

;   ***  to get or set the text in the edit box 
; at the top of a combo, use getText or setText


; == Fonts ==

loadFont proc szFontName:DWORD, fontSize:DWORD, attribs:DWORD
; --------------------------------------
; Returns a handle to the specified font
; --------------------------------------
;
; i.e.:
;
;   .data
;   myFontName db "Courier New",0
;
;   .code
;
;   push Bold+Italic+Strikeout
;   push 14
;   push OFFSET myFontName
;   call loadFont
;
;       or
;
;   push 0
;   push 10
;   push OFFSET myFontName
;   call loadFont
;
;      etc.
;
  push szFontName
  push DEFAULT_PITCH+FF_DONTCARE
  push PROOF_QUALITY
  push CLIP_DEFAULT_PRECIS
  push OUT_DEFAULT_PRECIS
  push DEFAULT_CHARSET
  
  mov eax,attribs
  mov ebx,eax
  and ebx,1
  push ebx    ; strikeout
  
  shr eax,1
  mov ebx,eax
  and ebx,1
  push ebx    ; underline
  
  shr eax,1
  mov ebx,eax
  and ebx,1
  push ebx    ; Italic
  
  shr eax,1
  and eax,1   ;(bold)
  test eax,eax
  jz @F
    add eax,299
  @@:
  add eax,400
  push eax    ; weight
  push 0
  push 0
  push 0
  push fontSize
  call CreateFont
  ret
loadFont endp


setFont proc hControl:DWORD, hFont:DWORD
  ; --------------------------------------
  ; Set the font for a control (or window)
  ; --------------------------------------
  push 1
  push hFont
  push WM_SETFONT
  push hControl
  call doSendMessage
  ret
setFont endp


doSendMessage proc arga:DWORD,argb:DWORD,argc:DWORD,argd:DWORD
  ; Windows API functions are 
  ; destructive to ecx and edx
  push ecx
  push edx
    push argd
    push argc
    push argb
    push arga
    call SendMessage
  pop edx
  pop ecx
  ret
doSendMessage endp





























