;-------------------------------------------------------------
; Simple module to edit the description string for a PUD file
;-------------------------------------------------------------




.data?

  hDescFont       dd  ?
  hDescButtonFont dd  ?
  hDescEditbox    dd  ?



;===============================================================================
;============================ Module definition ================================
;===============================================================================
;
; --> don't change this stuff apart from editing the title 
;     string and adding control ID containers to the list
;
Desc_trigger_id      dd  ?
Desc_hWindow         dd  ?

; You need to allocate a DWORD here for each control you create
;  on your module window, in the same order as you create them
;   regardless of whether you need to use their ID or not.

DescID_OkButton       dd  ?
DescID_CancelButton   dd  ?
DescID_MapdescText    dd  ?
DescID_EditBox        dd  ?

.data

 Desc_hModule   dd  OFFSET Desc_MODINFO   ; module handle

 szDescName     db "PUDed_Desc",0
 szDescTitle    db "Description",0

 Desc_MODINFO       dd  OFFSET szDescName       ; module name
		    dd  OFFSET szDescTitle      ; module display title
		    dd  OFFSET Desc_WndProc     ; module message processor
		    dd  OFFSET Desc_create      ; module constructor
		    dd  OFFSET Desc_trigger_id  ; module control ID list
		    dd  1                       ; module trigger sources
;
;===============================================================================



  szDescFontName      db "Times New Roman",0
  szMapDescription    db "Map Description:",0

  szDESC              db "DESC",0

  ok_button_text      db "Ok",0
  cancel_button_text  db "Cancel",0


.code

close_desc_window:
  push 0
  push Desc_hWindow
  call ShowWindow
  ret


fetch_pud_description:
  ; get handle to the loaded pud object
  mov edx,hCurrentPud
  
  ; if no file loaded return
  test edx,edx
  jnz @F
    ret
  @@:
  
  ; get handle to the "DESC" section
  mov ebx,OFFSET szDESC
  push [ebx]
  push edx
  call PUDfind_section
  
  ; set the editbox text
  push [eax+SECTION_DATA]
  push hDescEditbox
  call setText
  ret

update_pud_description:
  ; get handle to the loaded pud object
  mov edx,hCurrentPud
  
  ; if no file loaded return
  test edx,edx
  jnz @F
    ret
  @@:
  
  ;zero the start of the control string buffer to avoid
  ;copying garbage if it's a short description
  ;( DESC section is always 32 bytes )
  xor eax,eax
  mov ecx,8
  mov edi,OFFSET ControlStringBuffer
  @@:
    mov [edi],eax
    add edi,4
  loop @B
  
  push hDescEditbox
  call getText
  
  ; add a zero terminator in case it's > 31bytes
  add edi,3
  mov BYTE ptr [edi],0

  ; allocate memory for the new DESC section  
  push 32
  call doAlloc
  
  ; copy the first 32 bytes of the control string buffer to the new section
  mov edi,eax
  mov esi,OFFSET ControlStringBuffer
  mov ecx,32
  rep movsb

  push 32   ; arg 4 for PUDupdate_section
  push eax  ; arg 3 for PUDupdate_section ( PUDfind_section will desroy eax )
  
    ; get a handle to the DESC section
    mov ebx,OFFSET szDESC
    push [ebx]
    push edx
    call PUDfind_section
  
  ; update the DESC section
  push eax  ; arg 2
  push edx  ; arg 1
  call PUDupdate_section
  ret


Desc_WndProc proc WPhWnd:DWORD,WPuMsg:DWORD,WPwParam:DWORD,WPlParam:DWORD
  
  cmp WPuMsg, WM_COMMAND
  jnz Desc_NOT_CMD
    ; Command messages

    
    mov eax,DescID_CancelButton
    cmp eax,WPwParam
    jnz  @F
      ; code for "Cancel" Button
      ;just close the window
      call close_desc_window
      ret
    @@:
    mov eax,DescID_OkButton
    cmp eax,WPwParam
    jnz  @F
      ; code for "Ok" Button
      
      ; update "DESC" section with the contents of the textedit box
      
      call update_pud_description
      
      ;close the window
      call close_desc_window
      ret
    @@:
    
  
  Desc_NOT_CMD:
  
  ;std windows messages
  
  cmp WPuMsg, WM_ACTIVATE
  jnz @F
    ; the user has activated the module, 
    ; put the current map description in the textedit box
    
    call fetch_pud_description
    
    ret    
  @@:
    
  cmp WPuMsg, WM_DESTROY
  jnz @F
    ; the app is shutting down
    ; put any cleanup code here
    
    ret                    
  @@:
  
  ; unhandled message, pass to DefWindowProc
  push WPlParam
  push WPwParam
  push WPuMsg
  push WPhWnd
  call DefWindowProc
  ret
Desc_WndProc endp



Desc_create:
  
  ;------------------------------------
  ; initialize creation for this module
  ;------------------------------------
  push Desc_hModule
  call module_init
  
  ;---------------------
  ; create module window
  ;---------------------
  push 0
  push 0
  push 160              ; window height
  push 450              ; window width
  push 0
  push 0
  push OFFSET szDescTitle
  push 0
  call module_create
  
  
  ; load some fonts
  push Bold
  push 18
  push OFFSET szDescFontName
  call loadFont
  mov hDescButtonFont,eax
  
  push Italic
  push 24
  push OFFSET szDescFontName
  call loadFont
  mov hDescFont,eax
  
  
  
  ;---------------------
  ; create Ok button
  ;---------------------
  push 0
  push 0
  push 30               ; button height
  push 60               ; button width
  push 90               ; Y position
  push 265              ; X position
  push OFFSET ok_button_text
  push PUSHBUTTON
  call module_create
  
  push hDescButtonFont
  push eax
  call setFont
  
  ;---------------------
  ; create Cancel button
  ;---------------------
  push 0
  push 0
  push 30               ; button height
  push 80               ; button width
  push 90               ; Y position
  push 340              ; X position
  push OFFSET cancel_button_text
  push PUSHBUTTON
  call module_create
  
  push hDescButtonFont
  push eax
  call setFont
  
  
  ;------------------------
  ; create "Map Description" text
  ;------------------------
  push 0
  push 0
  push 25               ; height
  push 200              ; width
  push 20               ; Y position
  push 23               ; X position
  push OFFSET szMapDescription
  push TEXT
  call module_create
  
  push hDescButtonFont
  push eax
  call setFont
  
  
  ;------------------
  ; create a description text edit box
  ;------------------
  
  push 0
  push 0
  push 30               ; height
  push 400              ; width
  push 45               ; Y position
  push 20               ; X position
  push OFFSET no_text
  push TEXTEDIT
  call module_create
  
; (we want to use the handle for this one so save it)
  mov hDescEditbox,eax

  push hDescFont
  push eax
  call setFont
  

  ret














