;===============================================================================
;===                    pud object definition and access                     ===
;===============================================================================
;
;
; !* NOTE *!  Most of these procs are still completely untested and
;             may just crash or mess your stuff up.
;
;                       >>> PROCEED WITH CAUTION! <<<
;
;             ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
;       I'm quite sure it will all work how it is intended, but as most
;       of this lot still havn't been used there may still be a bug or 2
;       in there somewhere that will surface when we start using them.
;
;                       No big deal just fair warning :)
;
;
;
;   ==== Internal pud object format ====
;
;   --> linked list of section headers
;
;       where each section header is:
;
;          DWORD ptrNext     ( pointer to next header, 0 if last header )
;          DWORD name        ( the section name )
;          DWORD size        ( size of the section data )
;          DWORD ptrData     ( pointer to the section data )

;SECTION_NEXT    equ   0
 SECTION_NAME    equ   4
 SECTION_SIZE    equ   8
 SECTION_DATA    equ   12

SIZEOF_PUD_SECTION    equ 16



.code

doAlloc proc nBytes:DWORD
  ;## Global Alloc is destructive to ecx and edx ##
  push edx
  push ecx
    push nBytes
    push 0
    call GlobalAlloc
  pop ecx
  pop edx
  ret
doAlloc endp

doFree proc hMem:DWORD
  ;## Global Free is destructive to ecx and edx ##
  push edx
  push ecx
    push hMem
    call GlobalFree
  pop ecx
  pop edx
  ret
doFree endp




PUDdestroy proc hPud:DWORD
  ; deallocate a pud object
  
  cmp hPud,0
  jnz @F
    ret
  @@:
  mov eax,hPud
  free_next_hdr:
    
    ; is there section data storage allocated?
    mov ebx,[eax+SECTION_DATA]
    test ebx,ebx
    
    jz @F
    ;free section data storage
      push eax
	push ebx
	call doFree
      pop eax
    @@:
  
    ; save ptr to next header
    push [eax] 

      ;free section header
      push eax
      call doFree
  
    pop eax
    
    ; any more sections?
    test eax,eax
    jnz free_next_hdr
  
  ret
PUDdestroy endp

unlink_section proc hPud:DWORD, hSection:DWORD
  ; unlink section hSection from pud object hPud
  ;
  ; returns pud handle or 0 on fail
  ;
  ; ( hPud should be updated with the return value on
  ;      success as it will change if the first section is unlinked )
  
  mov eax, hPud
  cmp eax,hSection
  jne @F
    ; section is first
    ; second section handle is now hPud
    mov eax,[eax]
    ret
  @@:
  
  push ebx
  unlink_loop:
    ; get next hSection
    mov ebx,[eax]
    
    ; the one we want?
    cmp ebx,hSection
    jne @F
      
      ;get following hSection
      mov ebx,[ebx]
      
      ;over-write forward link in current section
      mov [eax],ebx
      
      ;return pud handle
      mov eax,hPud
      pop ebx
      ret
    
    @@:
    ;next section becomes current section
    mov eax,ebx
    
    ;end of pud object?
    test eax,eax
    jnz unlink_loop
  
  ; failed to find section
  pop ebx
  ret
unlink_section endp


PUDdelete_section proc hPud:DWORD, hSection:DWORD
  ; Delete section hSection from pud object hPud
  ;
  ; returns pud handle or 0 on fail
  ;
  ; ( hPud should be updated with the return value on
  ;      success as it will change if the first section is deleted )
  
  push hPud
  call PUDsection_count
  dec eax
  jnz @F
    ; the last remaining section cannot be deleted 
    ; ( use PUDdestroy )
    ret
  @@:
  
  ; unlink the section
  push hSection
  push hPud
  call unlink_section
  test eax,eax
  jnz @F
    ; failed
    ret
  @@:
  
  ; update pud object handle
  mov hPud,eax
  
  push ebx  ; preserve ebx
    
    ; is there section data storage allocated?
    mov eax,hSection
    mov ebx,[eax+SECTION_DATA]
    test ebx,ebx
    
    jz @F
      ; free section data storage
      push ebx
      call doFree
    @@:
    
    ; free section header
    push hSection
    call doFree
    
    ; return pud object handle
    mov eax,hPud
  
  pop ebx
  ret
PUDdelete_section endp


PUDfind_section proc hPud:DWORD, sName:DWORD
  ; returns a handle to the 
  ; named section or 0 if not found
  
  push ebx  ; preserve ebx
    mov eax,hPud
  
    @@:
      ; sName matches?
      mov ebx,[eax+SECTION_NAME]
      cmp ebx,sName
      je @F
	; get next section
	mov eax,[eax]
	
	; end of sections?
	test eax,eax
	jnz @B
    @@:
  
  pop ebx ; restore ebx
  ret
PUDfind_section endp

PUDupdate_section proc hPud:DWORD, hSection:DWORD, ptrNew:DWORD, nBytes:DWORD
  ; frees to old section data buffer and
  ; updates the contents of the section with
  ; the new data defined by ( ptrNew, nBytes )
  ;
  ; returns 0 on fail
    
  mov eax,hSection
    
  push [eax+SECTION_DATA]
  
  m2m [eax+SECTION_SIZE],nBytes
  m2m [eax+SECTION_DATA],ptrNew
  
  call doFree
  
  ret
PUDupdate_section endp

PUDsection_count proc hPud:DWORD
  ; return the number of sections in the pud object
  push esi
    mov esi,hPud
    xor eax,eax
    @@:
      inc eax
      mov esi,[esi]
      test esi,esi
      jnz @B
  pop esi
  ret
PUDsection_count  endp

PUDget_section proc hPud:DWORD,nSection:DWORD
  ;     (find section by index)
  ; return a handle to section number nSection
  ; or 0 on fail
  ;
  ; the first section is section 1
  
  push ecx  ; preserve ecx
    mov eax,hPud
    mov ecx,nSection
    dec ecx
    jnz @F
      ; we want the first section
      ; just return the pud handle
      pop ecx
      ret
    @@:
      ;get the next header
      mov eax,[eax]
      ;is it zero?
      ;( run out of sections )
      test eax,eax
      jz @F
	loop @B ; ecx-=1 ; jmp @B if ecx!=0
    @@:
  pop ecx
  ret
PUDget_section endp

PUDsection_index proc hPud:DWORD, sName:DWORD
  ; return the index of the named section
  ;  ( what number it is in the list )
  ; or 0 if not found
  ;
  ; the first section is section 1
  
  push esi  ; preserve esi
  push ebx  ; preserve ebx
    
    mov esi,hPud
    
    ;zero the count
    xor eax,eax
  
    @@:
      ; increase the count
      inc eax
      
      ; sName matches?
      mov ebx,[esi+SECTION_NAME]
      cmp ebx,sName
      je @F
	; next section
	mov esi,[esi]
	
	; no next section?
	test esi,esi
	jnz @B
	  ;not found
	  xor eax,eax
    @@:
  
  pop ebx ; restore ebx
  pop esi ; restore esi
  ret
PUDsection_index endp


PUDsection_set_index proc hPud:DWORD, hSection:DWORD, ndx:DWORD
  ; move to section hSection to position ndx in pud object hPud
  ;
  ; returns pud handle or 0 on fail
  ;
  ; ( hPud should be updated with the return value on
  ;      success as it will change if the first section is affected )
  push hSection
  push hPud
  call unlink_section
  test eax,eax
  jnz @F
    ret
  @@:
  
  ; update pud handle
  mov hPud,eax
  
  mov eax,ndx
  dec ndx
  test eax,eax
  jnz @F
    ;moving to index 1 - hSection is now hPud
    mov eax,hSection
    m2m [eax],hPud
    ret
  @@:
  
  ; get preceeding section
  push eax
  push hPud
  call PUDget_section
  
  ; save current link for preceeding section
  push [eax]
  
  ; overwrite with link to moved section
  m2m [eax],hSection
  
  ; link moved section to saved link
  mov eax,hSection
  pop [eax]
  
  ; return pud handle
  mov eax,hPud
  ret
PUDsection_set_index endp




;------------------------------------------------
; "new_section"
;------------------------------------------------
; This short subroutine just allocates space
; for a new section header. It takes no arguments.
; There is no need to use 'proc' to define it, we
; can just label the location, put a 'ret' at the
; end then just call that location.

new_section:
  
  push SIZEOF_PUD_SECTION
  call doAlloc
    
  test eax,eax
  jz @F
    ; ensure the pointers are initially 0
    mov DWORD ptr[eax],0
    mov DWORD ptr[eax+SECTION_DATA],0
  @@:
  ret
;------------------------------------------------


PUDcreate_section proc hPud:DWORD, sName:DWORD, ptrNew:DWORD, nBytes:DWORD
  ; creates a new section with the specified name
  ; and the data defined by ( ptrNew, nBytes )
  ; and adds to the hPud pud object
  ;
  ; returns handle to new section or 0 on fail
  
  LOCAL hLast
  
  push sName
  push hPud
  call PUDfind_section
  
  ; section already exists?
  test eax,eax
  jnz @F
    ; get handle to the last section
    push hPud
    call PUDsection_count
    
    push eax
    push hPud
    call PUDget_section
    
    mov hLast,eax 
    
    ;allocate a new header
    call new_section
    
    test eax,eax
    jz @F
      
      ; link the new header
      push edi  ; preserve edi
	
	mov edi,hLast
	mov [edi],eax
      
      pop edi
    
      ; fill the new header
      m2m [eax+SECTION_NAME],sName
      m2m [eax+SECTION_SIZE],nBytes
      m2m [eax+SECTION_DATA],ptrNew
      
      ;return new section handle
      ret
  @@:
  ; failed to create
  ; (section already exists or could not allocate header)
  xor eax,eax
  ret
PUDcreate_section endp








