; 
;   6) Add code to the module WndProc procedure in this file to
;      handle events and preform tasks for your module
;
;    ----------------------------------------------------------------------
;
;
;      A simple window/control creation framework has been 
;      added that so far supports the following controls:
;
;         PUSHBUTTON  -   A standard button on a window
;
;         TEXT        -   Some text on a window (a label)
;
;         TEXTEDIT    -   A box for text that the user can edit
;
;         CHECKBOX    -   A small square with a label that will
;                         add/remove a tick when the user clicks it
;
;         RADIOCHECK  -   A round checkbox that will uncheck any 
;                         other RADIOCHECK controls when it is checked
;
;         LISTBOX     -   A box containing a list of strings that 
;                         can be clicked by the user to highlight
;                         a particular string
;
;         COMBOBOX    -   A TEXTEDIT box with a drop-down LISTBOX
;                         attached to it so the user can select a string
;
;      There is also a few simple functions for interacting with 
;      controls in the .code section of PUDcontrol.inc
;
;   Your first call to "module_create" will automatically create a window
;   subsequecnt calls will create controls of the specified type on your
;   module window
;



RESHEADERSPACE    equ 10  ; SPACE AT TOP OF WINDOW
RESLABELH         equ 18  ; HEIGHT OF LABELS
RESBOXH           equ 20  ; HEIGHT OF EACH RES TEXT EDIT BOX
RESBOXPADH        equ 5   ; VERTICAL SPACE BETWEEN RES BOXES

RESBOXPADW        equ 5   ; HORIZONTAL SPACE BETWEEN RES BOXES
RESLABW           equ 60  ; WIDTH OF "Player #" LABELS
RESBOXW           equ 70  ; WIDTH OF RES EDIT TEXT BOXES

RESOKBUTH         equ 50  ; HEIGHT OF OK BUTTON FOR RES WINDOW


;=======================================================
;                     THESE ARE EQUATES
;=======================================================
;
;     NAME      equ     (value)
;
; Makes the compiler equate the label 'NAME' with (value)
; These lines do not compile to anything: they do not 
; result in any ML instructions ( here where that are declare)
;
; Anywhere following where immediate values may be written
; into the .code or .data sections (most commonly) if 
;'NAME' is included in the code, the compiler will substitute
; (value) before compiling the instruction or data item.
;
;   i.e.    MyFOOBAR equ 123h
;   ;  --     --     --     --    --
;
;   .code
;   mov eax,MyFOOBAR
;
; In this case the instruction will be converted into: 
;
;   mov eax,123h
;
; Then the instruction is normally compiled.
; 
; Equates are a way of making source code easier to decipher
; and maintain. They are constant values,that can also be 
; calculated at compile time from multiple values combined 
; with standard mathematical and logic operators.
;
;                           i.e.:    +,-,/,*,AND,OR,etc..
;
;==========================================================
;
;  This set of equates have been set up so that the  values
;  in the first group of equates could be adjusted until the
;  window and res boxes looked appropriate.
;
;                 ---     ---     ---
;
;   The second group of equates are calculated from the
;   calculated from the values in the first group, it 
;   is mostly this second group that is used 
;
;                 ---     ---     ---
;     
;   You can change the values of the top group of equates 
;   then re-compile the program, and you will see the size 
;   and layout of the starting resources window change
;   according to the values you have set.
;   
;                 ---     ---     ---
;
;   In the end the program compiles with these equates in
;   an identical fashion to that which would have been 
;   produced if the equivalent immediate numerical values
;   were written directly into the appropriate instructions.
;
;==========================================================



;==========================================================


RESGLOLABW    equ (RESBOXADDX*3)-RESBOXPADW ; Width of "Gold Lumber Oil" label

RESBOXOSX     equ RESBOXPADW + RESLABW  ; X offset of Gold boxes
RESBOXADDX    equ RESBOXW + RESBOXPADW  ; X offest add for next box on line

RESBOXOSY     equ RESHEADERSPACE + RESLABELH  ; Y offset of Player 1 res boxes
RESBOXADDY    equ RESBOXH + RESBOXPADH        ; add to Y offset for each player

RESWINH       equ (RESBOXADDY*9)+RESBOXOSY+RESOKBUTH+RESBOXPADH  ; res window height
RESWINW       equ (RESBOXADDX*3)+RESBOXOSX+RESBOXPADW ; res window width

;==============================================================
PUD_GOLD    equ 1
PUD_LUMBER  equ 2
PUD_OIL     equ 3










;==============================================================
;==============================================================
;===================== .data? section =========================
;==============================================================
;====================Un-Initialised Data=======================
;==============================================================
;==============================================================

.data?
  hLabelFont    dd  ?
  hHeadingFont  dd  ?
  hResBox       dd 18h dup(?) ; handles for 24 res edit text boxes
			      ; Player1 Gold, Lumber, Oil, Player2 Gold,etc...
  boxbuf        db 256 dup(?)



;===============================================================================
;============================ Module definition ================================
;===============================================================================
;
; --> don't change this stuff apart from editing the title 
;     string and adding control ID containers to the list
;
startres_trigger_id      dd  ?
startres_hWindow         dd  ?

; You need to allocate a DWORD here for each control you create
;  on your module window, in the same order as you create them
;   regardless of whether you need to use their ID or not.

startresID_GLOLabel       dd  ?
startresID_DupButton      dd  ?
startresID_resboxes       dd 20h dup(?)   ; 24 res boxes - 3 x 8 players
					  ; +8 "Player #" labels = 32 DWORDs
startresID_OkButton       dd ?
startresID_CancelButton   dd ?



;==============================================================
;====================== .data section =========================
;==============================================================
;===================== Initialised Data =======================
;==============================================================

.data

 startres_hModule   dd  OFFSET startres_MODINFO   ; module handle

 szstartresName     db "PUDed_startres",0
 szstartresTitle    db "Starting Resources",0

 startres_MODINFO   dd  OFFSET szstartresName       ; module name
		    dd  OFFSET szstartresTitle      ; module display title
		    dd  OFFSET startres_WndProc     ; module message processor
		    dd  OFFSET startres_create      ; module constructor
		    dd  OFFSET startres_trigger_id  ; module control ID list
		    dd  1                           ; module trigger sources

;
;===============================================================================


  sZduplicate     db "Duplicate",0
  sZresplayerlab  db " Player "
  sZresplayernum  db "0",0
  sZGoldLumberOil db "   Gold      Lumber      Oil",0
  sZok            db "Ok",0
  sZcancel        db "Cancel",0

  ; *these 3 don't actually need the ,0 but might be used
  ; later for section clipboard display strings so it won't
  ; hurt to zero-terminate them now to save messing around later.
  szSGLD          db "SGLD",0
  szSLBR          db "SLBR",0
  szSOIL          db "SOIL",0


.code


box_get_val proc hBox:DWORD
  ; get the decimal value of the string
  ; in resbox hBox
  
  ; also limits to 0-65535 and 
  ; rewrites the string in the box

LOCAL retval
  
  pushad
  mov esi,OFFSET boxbuf
  mov BYTE ptr[esi],0
  
  ; get the string in the text box
  push esi
    push 32
    push esi
    push hBox
    call GetWindowText
  pop esi
  
  ; convert it to an integer value
  push esi
  call getDecimal
  
  ; limit the value to the legal range
  push 65535
  push 0
  push eax
  call limit
  
  ; set as the return value from this proc
  mov retval,eax
  
  ; write back to the edit box
  push eax
  push hBox
  call box_set_val

  popad
  ; get return value
  mov eax,retval
  
  ret
box_get_val endp


box_set_val proc hBox:DWORD,val:DWORD
  ; set the text in box hBox to the decimal
  ; string equivalent of val
  pushad

    ; write the value as a string
    push val
    push OFFSET boxbuf
    call strDecimal

    ; put the string into the edit box
    push OFFSET boxbuf
    push hBox
    call SetWindowText

  popad
  ret
box_set_val endp


startres_get_handle proc player:DWORD,res:DWORD
  ; get the handle of res box for player/res from the array
  ; res: gold=1 wood=2 oil=3
  push esi
  push ebx
  mov eax,player
  dec eax             ; -1
  shl eax,2           ; x4
  lea eax,[eax+eax*2] ;(x4 + x8) = x12
  lea esi,[eax+OFFSET hResBox]
  
  mov eax,res
  dec eax
  lea esi,[esi+eax*4]
  mov eax,[esi]
  pop ebx
  pop esi
  ret
startres_get_handle endp


startres_get proc player:DWORD,res:DWORD
  ; get the value of the string in the text
  ; edit box for player/res
  push res
  push player
  call startres_get_handle
  
  push eax
  call box_get_val
  ret
startres_get endp

startres_set proc player:DWORD,res:DWORD,val:DWORD
  ; set the the string in the text edit box 
  ; for player/res to equal value val
  push res
  push player
  call startres_get_handle
  
  push val
  push eax
  call box_set_val
  ret
startres_set endp

pud_get_res proc hPud:DWORD,res:DWORD
  ; get starting resource values for 
  ; resource res from loaded PUD hPud 
  ; and put them in the edit boxes
  pushad
  
  ; get (res-1)*5
  mov eax,res
  dec eax
  lea eax,[eax+eax*4]
  
  ; find section 4CC
  mov esi,OFFSET szSGLD
  add esi,eax
  
  ;get handle to the starting res section (SGLD,SLBR or SOIL)
  push [esi]
  push hPud
  call PUDfind_section
  
  ;locate the starting res section data
  mov esi,[eax+SECTION_DATA]
  
  ;start at player 8 (WORD values)
  add esi,16  ; (8*2=16)
  
  mov ecx,8
  @@:
    sub esi,2
    
    ; get the WORD res value
    movzx eax,WORD ptr[esi]
    
    ; set the res box text
    push eax    ; val
    push res    ; res
    push ecx    ; player
    call startres_set
  loop @B
  
  popad
  ret
pud_get_res endp

pud_put_res proc hPud:DWORD,res:DWORD
  ; get starting resource values for 
  ; resource res from the edit boxes 
  ; and write them to loaded PUD hPud
  pushad
  
  ; get (res-1)*5
  mov eax,res
  dec eax
  lea eax,[eax+eax*4]
  
  ; find section 4CC
  mov esi,OFFSET szSGLD
  add esi,eax
  
  ;get handle to the starting res section (SGLD,SLBR or SOIL)
  push [esi]
  push hPud
  call PUDfind_section
  
  ;locate the starting res section data
  mov esi,[eax+SECTION_DATA]
  
  ;start at player 8 (WORD values)
  add esi,16  ; (8*2=16)
  
  mov ecx,8
  @@:

    ; get the res box value
    push res    ; res
    push ecx    ; player
    call startres_get

    sub esi,2
    
    ; set the WORD res value
    mov [esi],ax
    
  loop @B
  
  popad
  ret
pud_put_res endp


zero_startres:
  ; zero all starting res boxes
  push esi
  push ecx
  
  mov esi, OFFSET hResBox
  mov ecx,24
  @@:
    push 0
    push [esi]
    call box_set_val
    add esi,4
  loop @B

  pop ecx
  pop esi
  ret



pud_get_startres proc hPud:DWORD
  ; get starting resource values from loaded 
  ; PUD hPud and put them in the edit boxes
  cmp hPud,0
  jnz @F
    ; no current PUD handle
    call zero_startres
    ret
  @@:
  push ecx
  mov ecx,PUD_OIL
  @@:
    push ecx
    push hPud
    call pud_get_res
  loop @B
  pop ecx
  ret
pud_get_startres endp

pud_put_startres proc hPud:DWORD
  ; get starting resource values from the edit
  ; boxes and write them to loaded PUD hPud
  cmp hPud,0
  jnz @F
    ret
  @@:
  push ecx
  mov ecx,PUD_OIL
  @@:
    push ecx
    push hPud
    call pud_put_res
  loop @B
  pop ecx
  ret
pud_put_startres endp


do_startres_dup proc
  ; duplicate player 1 values for players 2-8
  push edx
  push ecx
  
  mov edx,PUD_OIL   ; edx=current res
  
  dup_loop:
  
    ;get player 1 value
    push edx
    push 1
    call startres_get 
  
    mov ecx,7 ; setting for 7 players
  
    @@:
      ; (save the value)
      push eax
      
	; the value
	push eax
  
	; the resource
	push edx
  
	; the player
	inc ecx
	push ecx
	dec ecx
	call startres_set
    
      ; (retreive the value)
      pop eax
    loop @B
    
    dec edx
  jnz dup_loop
  
  pop ecx
  pop edx
  ret
do_startres_dup endp

startres_WndProc proc WPhWnd:DWORD,WPuMsg:DWORD,WPwParam:DWORD,WPlParam:DWORD
  
  cmp WPuMsg, WM_COMMAND
  jnz startres_NOT_CMD
    ; Command messages

    
    mov eax,startresID_CancelButton
    cmp eax,WPwParam
    jnz  @F
      ;close the window
      push 0
      push startres_hWindow
      call ShowWindow
      ret
    @@:
    
    mov eax,startresID_OkButton
    cmp eax,WPwParam
    jnz  @F
      ;update the PUD starting res 
      push hCurrentPud
      call pud_put_startres
      
      ;close the window
      push 0
      push startres_hWindow
      call ShowWindow
      ret
    @@:
    
    mov eax,startresID_DupButton
    cmp eax,WPwParam
    jnz  @F
      call do_startres_dup
      ret
    @@:
    
  startres_NOT_CMD:
  
  ;std windows messages
  cmp WPuMsg, WM_ACTIVATE
  jnz @F
    ; the user has activated this module, 
    ; prepare display state here.
    push hCurrentPud
    call pud_get_startres
    ret    
  @@:
    
  ; unhandled message, pass to DefWindowProc
  push WPlParam
  push WPwParam
  push WPuMsg
  push WPhWnd
  call DefWindowProc
  ret
startres_WndProc endp



startres_create:
  
  ;------------------------------------
  ; initialize creation for this module
  ;------------------------------------
  push startres_hModule
  call module_init
  
  ;---------------------
  ; create module window
  ;---------------------
  push 0
  push 0
  push RESWINH    ; window height
  push RESWINW    ; window width
  push 0
  push 0
  push OFFSET szstartresTitle
  push 0
  call module_create
  
  
  ; load some fonts
  ; ---------------
  
   ; Label Font
  push 0
  push 14
  push OFFSET szArial
  call loadFont
  mov hLabelFont,eax
  
   ; Heading Font
  push Bold
  push 15
  push OFFSET szCourierNew
  call loadFont
  mov hHeadingFont,eax
  
  
  ; create "Gold  Lumber  Oil" label
  ; --------------------------------
  push 0
  push 0
  push RESLABELH        ; height
  push RESGLOLABW       ; width
  push RESHEADERSPACE   ; Y position
  push RESBOXOSX        ; X position
  push OFFSET sZGoldLumberOil
  push TEXT
  call module_create
  
  ; set the label's font
  push hHeadingFont
  push eax
  call setFont

  
  ; create "Duplicate" button
  ; -------------------------
  push 0
  push 0
  push RESLABELH                ; button height
  push RESLABW-RESBOXPADW       ; button width
  push RESHEADERSPACE-RESBOXPADH; Y position
  push RESBOXPADW               ; X position
  push OFFSET sZduplicate
  push PUSHBUTTON
  call module_create
  
  ; don't need the handle for a pushbutton 
  ; - command messages use the control's ID 
  ; (they are sent to the parent window)
  
  push hLabelFont
  push eax
  call setFont


  ;========================================
  ; Create player labels and resource boxes 
  ;========================================
  
  ; Init
  mov ecx,8                 ; Number of players left to create boxes for
  mov edx,RESBOXOSY         ; Y position on window
  mov edi, OFFSET hResBox   ; ptr to handle storage array current address


  ; -------------------
  ;>> for each player <<
  ; -------------------
  startres_create_loop:
  
  
    ; update the "Player #" string
    inc sZresplayernum
  
    ; create the "Player #" label
    push 0
    push 0
    push RESLABELH        ; height
    push RESLABW          ; width
    push edx              ; Y position
    push RESBOXPADW       ; X position
    push OFFSET sZresplayerlab
    push TEXT
    call module_create
  
    ; set the label's font
    push hLabelFont
    push eax
    call setFont
  
    ; set the starting X position on window
    mov ebx,RESBOXOSX   
  
  
    push ecx      ; save number of players left
      
      mov ecx,3     ; number of boxes left to create for this player
  
      @@:
	; create a starting res edit text box
	push 0
	push 0
	push RESBOXH          ; height
	push RESBOXW          ; width
	push edx              ; Y position
	push ebx              ; X position
	push OFFSET no_text
	push TEXTEDIT
	call module_create
  
	; save the handle in the res box handle array
	mov [edi],eax
    
	;inc handle array pointer
	add edi,4
  
	;inc the X position
	add ebx,RESBOXADDX
    
      ;decrease box count for this player (ecx) and loop if ecx != 0
      loop @B
  
    pop ecx   ; retrieve number of players left
  
    ;inc the Y position
    add edx,RESBOXADDY
  
  ;decrease players left count (ecx) and loop if ecx != 0
  loop startres_create_loop

  ; create Ok button
  ;-------------------
  push 0
  push 0
  push RESOKBUTH                            ; button height
  push RESOKBUTH+(RESBOXPADW*2)             ; button width
  push RESWINH-RESOKBUTH-(RESBOXPADH*6)     ; Y position
  push RESWINW-RESOKBUTH-(RESBOXPADW*4)     ; X position
  push OFFSET sZok
  push PUSHBUTTON
  call module_create

  ; don't need the handle for a pushbutton 
  ; - command messages use the control's ID 
  ; (they are sent to the parent window)

  push hDescButtonFont
  push eax
  call setFont


;                                   *>Here seemed like a good place 
;                                   for a discussion on equates.

;                                   This is general ASM information 
;                                   and not directly related to the 
;                                   PUD editor.
;
;                                   Read if you are interested or
;                                   having difficulty using equates, 
;                                   otherwise 
;                                              this can be skipped.
;                                             ~~~~~~~~~~~~~~~~~~~~~~
;######################  #############################################
;
; Even here where the  /\  equates are entered into the actual
; instruction source  /  \  we can still do all sorts of math
; operations on them   ||   as this is all worked out by the
; the compiler when it || converts the instructions into ML code.
;                      ||
;
;
;        ---    ---    ---    ---    ---    ---    
;
;
;      *+*+*=->   It must be noted that the result of these math
; operations must always remain a constant value; i.e. When the 
; instruction is compiled it will reduce to a single immediate 
; value that is coded into that instruction and never changes. 
; These math operations are NOT done when the program is running
; so they cannot be used with values in volatile containers like
; registers or memory.  i.e.:
;
;
;   You can NOT do this:
;
;     FOO   equ 12
;     BAR   equ 1Eh
;
;    .code
;
;     add eax,ebx-(BAR+FOO)     ;   <-- Error in instruction
;
;
;
;     In this case, the compiler cannot calculate the value 
; of the whole right side (SRC) of the instruction, because
; it contains ebx. The value contained in ebx at compile time 
; will be some value being used by the compiler itself and
; have nothing to do with the program being compiled. To effect 
; the result alluded to in the above instruction, the math 
; operation on the register (in this case a subtraction), must 
; be done using an opcode, so this would have to be split into 
; two seperate instructions; i.e.:
;
;
;   sub ebx, BAR+FOO
;   add eax, ebx
;
;
; As the SRC operand for the sub instruction now exclusively
; contains equate values, BAR+FOO is easily evaluated to a 
; constant 42, so the resulting instructions that would be
; compiled are:   
;
;
;   sub ebx, 42
;   add eax, ebx
;
;
; * This produces the intended result in eax, noting that the
; value in ebx has been changed, however for the purposes of
; the example the ebx value after these instructions is assumed
; to be discarded and not relevant.
;
;
;        ---    ---    ---    ---    ---    ---    
;
;
; EXCEPTION: (sort of)
;
; The following example IS correct and will NOT produce a 
; compiler error:
;
; 
;     FOO   equ   02C45A4h
;     BAR   equ   8
;
;     .code
;
;     lea eax,[ esi+FOO+ebx*BAR ]
;
;
;-> This is acceptable because in this case the '+' and '*' 
;-> operators are part of the dereferencing mechanism ( being
;-> inside the [] square brackets ) and have nothing to do with 
;-> the previously discussed math operators except that they
;-> are represented by the same ASCII characters.
;
; This will compile as:
;
;
;     lea eax,[ esi + 02C45A4h + ebx*8  ]
;
;
;#####################################################################


  ; create Cancel button
  ;----------------------
  push 0
  push 0
  push RESOKBUTH-(RESBOXPADH*4)             ; button height
  push RESOKBUTH+(RESBOXPADW*5)             ; button width
  push RESWINH-RESOKBUTH-(RESBOXPADH*2) ; Y position
  push RESBOXPADW 
  push OFFSET sZcancel
  push PUSHBUTTON
  call module_create


  ; don't need the handle for a pushbutton 
  ; - command messages use the control's ID 
  ; (they are sent to the parent window)

  push hDescButtonFont
  push eax
  call setFont
  
  ret














