;------------------------------------
;-- Template For PUDed.exe modules --
;------------------------------------
;
;   How to use this to create a module:
;     -----------------------------
;
;   1) Decide on a name for your module - something simple with no spaces
;      maybe "unitprop" for a unit properties editor or something
;
;   2) Make a copy of this file and rename it "PUDunitprop.inc"
;      or whatever your module name is.
;
;   3) Do a text search & replace for the string "template" and 
;      replace it with your name.
;
;   4) Everything between the "DEMO CODE" and "/DEMO CODE" comments 
;      can be deleted however to start with you may want to leave it 
;      there and just alter it to suit your needs.
;
;   5) Add module_create calls to the create subroutine at the end of
;      this file to create your module window and the controls on it
;     
;      Make sure you add a 'dd' declare for each control where indicated 
;      in the .data? section.
; 
;   6) Add code to the module WndProc procedure in this file to
;      handle events and preform tasks for your module
;
;   7) in the PUDed.asm main file add a "include PUDunitprop.inc" line
;      and register the module by adding:
;
;               push unitprop_hModule         (or whatever)
;               call register_module
;
;      where indicated.
;
;
;    ----------------------------------------------------------------------
;
;
;      A simple window/control creation framework has been 
;      added that so far supports the following controls:
;
;         PUSHBUTTON  -   A standard button on a window
;
;         TEXT        -   Some text on a window (a label)
;
;         TEXTEDIT    -   A box for text that the user can edit
;
;         CHECKBOX    -   A small square with a label that will
;                         add/remove a tick when the user clicks it
;
;         RADIOCHECK  -   A round checkbox that will uncheck any 
;                         other RADIOCHECK controls when it is checked
;
;         LISTBOX     -   A box containing a list of strings that 
;                         can be clicked by the user to highlight
;                         a particular string
;
;         COMBOBOX    -   A TEXTEDIT box with a drop-down LISTBOX
;                         attached to it so the user can select a string
;
;      There is also a few simple functions for interacting with 
;      controls in the .code section of PUDcontrol.inc
;
;   Your first call to "module_create" will automatically create a window
;   subsequecnt calls will create controls of the specified type on your
;   module window
;




.data?
; ===        === DEMO CODE ===       === 
  hSmallFont    dd  ?
  hBigFont      dd  ?
  hFancyFont    dd  ?
  
  hMyCheckbox   dd  ?
  hMyEditbox    dd  ?
; ===        ===/DEMO CODE ===       === 



;===============================================================================
;============================ Module definition ================================
;===============================================================================
;
; --> don't change this stuff apart from editing the title 
;     string and adding control ID containers to the list
;
template_trigger_id      dd  ?
template_hWindow         dd  ?

; You need to allocate a DWORD here for each control you create
;  on your module window, in the same order as you create them
;   regardless of whether you need to use their ID or not.

templateID_ThisButton     dd  ?
templateID_ThatCheckbox   dd  ?
templateID_ATextLabel     dd  ?
templateID_TheEditBox     dd  ?

.data

 template_hModule   dd  OFFSET template_MODINFO   ; module handle

 sztemplateName     db "PUDed_template",0
 sztemplateTitle    db "Demo Module",0

 template_MODINFO   dd  OFFSET sztemplateName       ; module name
		    dd  OFFSET sztemplateTitle      ; module display title
		    dd  OFFSET template_WndProc     ; module message processor
		    dd  OFFSET template_create      ; module constructor
		    dd  OFFSET template_trigger_id  ; module control ID list
		    dd  1                           ; module trigger sources
;
;===============================================================================


; ===        === DEMO CODE ===       === 

  button_text   db "Ok",0
  checkbox_text db "Show My Text",0
  static_text   db "Write some text in here:",0
  no_text       db 0

  szArial       db "Arial",0
  szCourierNew  db "Courier New",0
  szTimesNR     db "Times New Roman",0
; ===        ===/DEMO CODE ===       === 



.code


template_WndProc proc WPhWnd:DWORD,WPuMsg:DWORD,WPwParam:DWORD,WPlParam:DWORD
  
  cmp WPuMsg, WM_COMMAND
  jnz template_NOT_CMD
    ; Command messages

    
    mov eax,templateID_ThisButton
    cmp eax,WPwParam
    jnz  @F
      ; code for ThisButton here
      
; ===        === DEMO CODE ===       === 
      ;close the window
      push 0
      push template_hWindow
      call ShowWindow
      
      ;is the checkbox checked?
      push hMyCheckbox
      call getCheck
      test eax,eax
      jz dont_show_text
	
	; get the text from the edit box
	push hMyEditbox
	call getText
	
	; display the text
	push 0
	push OFFSET sztemplateTitle
	push eax
	push 0
	call MessageBoxA
      
      dont_show_text:
; ===        ===/DEMO CODE ===       === 

      ret
    @@:
    
    mov eax,templateID_ThatCheckbox
    cmp eax,WPwParam
    jnz  @F
      ; code for ThatCheckbox here
      ;
      ;( we dont need to do anything
      ;       when the user clicks the checkbox)
      ret
    @@:
    
  
  template_NOT_CMD:
  
  ;std windows messages
  cmp WPuMsg, WM_CREATE
  jnz @F
    ; window has been created, 
    ; initialization code here.
    
    mov eax,1
    ret    
  @@:
  
  cmp WPuMsg, WM_ACTIVATE
  jnz @F
    ; the user has activated this module, 
    ; prepare display state here.
    
    ;ret    
  @@:
    
  cmp WPuMsg, WM_DESTROY
  jnz @F
    ; the app is shutting down
    ; put any cleanup code here
    
    ret                    
  @@:
  
  ; unhandled message, pass to DefWindowProc
  push WPlParam
  push WPwParam
  push WPuMsg
  push WPhWnd
  call DefWindowProc
  ret
template_WndProc endp



template_create:
  
  ;------------------------------------
  ; initialize creation for this module
  ;------------------------------------
  push template_hModule
  call module_init
  
  ;---------------------
  ; create module window
  ;---------------------
  push 0
  push 0
  push 300              ; window height
  push 450              ; window width
  push 0
  push 0
  push OFFSET sztemplateTitle
  push 0
  call module_create
  
; ===        === DEMO CODE ===       === 
  
  ; load some fonts for lols
  push 0
  push 12
  push OFFSET szArial
  call loadFont
  mov hSmallFont,eax
  
  push Bold
  push 28
  push OFFSET szCourierNew
  call loadFont
  mov hBigFont,eax
  
  push Italic+Underline
  push 20
  push OFFSET szTimesNR
  call loadFont
  mov hFancyFont,eax
  
  
  
  ;---------------------
  ; create a push-button
  ;---------------------
  push 0
  push 0
  push 40               ; button height
  push 80               ; button width
  push 220              ; Y position
  push 345              ; X position
  push OFFSET button_text
  push PUSHBUTTON
  call module_create
  
  ; don't need the handle for a pushbutton 
  ; - command messages use the control's ID 
  ; (they are sent to the parent window)
  
  push hBigFont
  push eax
  call setFont
  
  ;-------------------
  ; create a check-box
  ;-------------------
  push 0
  push 0
  push 25               ; height
  push 120              ; width (includes text)
  push 200              ; Y position
  push 50               ; X position
  push OFFSET checkbox_text
  push CHECKBOX
  call module_create
  
; (we want to use the handle for this one so save it)
  mov hMyCheckbox,eax
  
  push hSmallFont
  push eax
  call setFont
  
  
  ;------------------------
  ; create some static text
  ;------------------------
  push 0
  push 0
  push 25               ; height
  push 200              ; width
  push 125              ; Y position
  push 30               ; X position
  push OFFSET static_text
  push TEXT
  call module_create
  
  
  push hFancyFont
  push eax
  call setFont
  
  
  ;------------------
  ; create a text box
  ;------------------
  
  push 0
  push 0
  push 25               ; height
  push 200              ; width
  push 150              ; Y position
  push 40               ; X position
  push OFFSET no_text
  push TEXTEDIT
  call module_create
  
; (we want to use the handle for this one so save it)
  mov hMyEditbox,eax

; ===        ===/DEMO CODE ===       === 
  

  ret














